<?php

namespace Realestate\Plan\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Realestate\Plan\Http\Requests\SubscriptionUsageRequest;
use Realestate\Plan\Interfaces\SubscriptionUsageRepositoryInterface;
use Realestate\Plan\Models\SubscriptionUsage;

/**
 * Resource controller class for subscription_usage.
 */
class SubscriptionUsageResourceController extends BaseController
{

    /**
     * Initialize subscription_usage resource controller.
     *
     * @param type SubscriptionUsageRepositoryInterface $subscription_usage
     *
     * @return null
     */
    public function __construct(SubscriptionUsageRepositoryInterface $subscription_usage)
    {
        parent::__construct();
        $this->repository = $subscription_usage;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Realestate\Plan\Repositories\Criteria\SubscriptionUsageResourceCriteria::class);
    }

    /**
     * Display a list of subscription_usage.
     *
     * @return Response
     */
    public function index(SubscriptionUsageRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Realestate\Plan\Repositories\Presenter\SubscriptionUsagePresenter::class)
                ->$function();
        }

        $subscription_usages = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('plan::subscription_usage.names'))
            ->view('plan::subscription_usage.index', true)
            ->data(compact('subscription_usages', 'view'))
            ->output();
    }

    /**
     * Display subscription_usage.
     *
     * @param Request $request
     * @param Model   $subscription_usage
     *
     * @return Response
     */
    public function show(SubscriptionUsageRequest $request, SubscriptionUsage $subscription_usage)
    {

        if ($subscription_usage->exists) {
            $view = 'plan::subscription_usage.show';
        } else {
            $view = 'plan::subscription_usage.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('plan::subscription_usage.name'))
            ->data(compact('subscription_usage'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new subscription_usage.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(SubscriptionUsageRequest $request)
    {

        $subscription_usage = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('plan::subscription_usage.name')) 
            ->view('plan::subscription_usage.create', true) 
            ->data(compact('subscription_usage'))
            ->output();
    }

    /**
     * Create new subscription_usage.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SubscriptionUsageRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $subscription_usage                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('plan::subscription_usage.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/subscription_usage/' . $subscription_usage->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/plan/subscription_usage'))
                ->redirect();
        }

    }

    /**
     * Show subscription_usage for editing.
     *
     * @param Request $request
     * @param Model   $subscription_usage
     *
     * @return Response
     */
    public function edit(SubscriptionUsageRequest $request, SubscriptionUsage $subscription_usage)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('plan::subscription_usage.name'))
            ->view('plan::subscription_usage.edit', true)
            ->data(compact('subscription_usage'))
            ->output();
    }

    /**
     * Update the subscription_usage.
     *
     * @param Request $request
     * @param Model   $subscription_usage
     *
     * @return Response
     */
    public function update(SubscriptionUsageRequest $request, SubscriptionUsage $subscription_usage)
    {
        try {
            $attributes = $request->all();

            $subscription_usage->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('plan::subscription_usage.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/subscription_usage/' . $subscription_usage->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/subscription_usage/' . $subscription_usage->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the subscription_usage.
     *
     * @param Model   $subscription_usage
     *
     * @return Response
     */
    public function destroy(SubscriptionUsageRequest $request, SubscriptionUsage $subscription_usage)
    {
        try {

            $subscription_usage->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::subscription_usage.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('plan/subscription_usage/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/subscription_usage/' . $subscription_usage->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple subscription_usage.
     *
     * @param Model   $subscription_usage
     *
     * @return Response
     */
    public function delete(SubscriptionUsageRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::subscription_usage.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('plan/subscription_usage'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/subscription_usage'))
                ->redirect();
        }

    }

    /**
     * Restore deleted subscription_usages.
     *
     * @param Model   $subscription_usage
     *
     * @return Response
     */
    public function restore(SubscriptionUsageRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('plan::subscription_usage.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/plan/subscription_usage'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/subscription_usage/'))
                ->redirect();
        }

    }

}
